/* 
   Light LEDs in a specific order/pattern
   Up to 64bit 64 LEDs Version
   
   Sketch used in
   2018-09-26 "Thermometer" Style LED Blinking  http://forum.arduino.cc/index.php?topic=570493.msg0#new
   2018-07-19 Lauflicht in zwei Richtungen rund um den  http://forum.arduino.cc/index.php?topic=558863.0
   Ähnlich dem Muster von http://forum.arduino.cc/index.php?topic=518131.0
   https://forum.arduino.cc/index.php?topic=711402.0
   2021-09-20 Multiple led sequence over time https://forum.arduino.cc/t/multiple-led-sequence-over-time/907021/67
   
   by noiasca
   Latest sketch available on: https://werner.rothschopf.net/microcontroller/202109_millis_led_sequence_en.htm
*/

/* ***************************
    Configuration/Konfiguraton
 * ************************* */
// the pattern is defined in an separate file/tab
#include "pattern16.h"    // basic example for up to 16 LEDs
//#include "pattern64.h"    // example for a large 64bit wide array for up to 64 LEDs - obviously only for an Arduino Mega

const uint8_t ledPin[] = {13, 2, 3, 4, 5, 6, 7};                     // some reduced pin definitions for my test board
//const uint8_t ledPin[] = {13, 12, 10, 9, 8, 7, 6, 5, 4, 3, 2, A0, A1, A2, A3, A4, 13, 12, 10, 9, 8, 7, 6, 5, 4, 3, 2, A0, A1, A2, A3, A4, 13, 12, 10, 9, 8, 7, 6, 5, 4, 3, 2, A0, A1, A2, A3, A4, 13, 12, 10, 9, 8, 7, 6, 5, 4, 3, 2, A0, A1, A2, A3, A4}; // currently maximum are 64 pins - first pin is index 0 -> the LSB (the right most) bit in the pattern
const uint16_t interval = 100;                                       // hint: if you never exceed 254ms you can use an uint8_t also
#define DEBUG_UART 1                                                 // activate debug output on Serial to see the LED pattern 

/* ***************************
    Global variables 
 * ************************* */
const size_t totalNoPattern = sizeof(pattern) / sizeof(pattern[0]);   // how many patterns are defined
const size_t totalNoPins = sizeof(ledPin) / sizeof(ledPin[0]);        // how many pins are defined

/* ***************************
    Sequence
    needs to be called in loop as often as possible.
 * ************************* */

void runSequence()
{
  static uint32_t previousMillis = 0;                  // last update of the LED pattern
  static size_t actualPattern = 0;                     // number of actual active pattern
  if (millis() - previousMillis >= interval)           // time management based on "Blink without Delay"
  {
    // lets copy the actual pattern from progmem to a temporary local variable in SRAM
    patternType temp;
    memcpy_P(&temp, &pattern[actualPattern], sizeof(patternType));
    // now check each bit for each LED if it is 0 or 1 and switch the pin accordingly
    for (size_t i = 0; i < totalNoPins; i++)
    {
      if (temp & ((patternType)1 << i)) {           // cast the 1 to the size of the pattern before the shift, important for 32bit and 64bit pattern on AVRs with 16bit integers
#if DEBUG_UART
        Serial.print(F("X"));
#endif
        digitalWrite(ledPin[i], HIGH);
      }
      else {
#if DEBUG_UART
        Serial.print(F(" "));
#endif
        digitalWrite(ledPin[i], LOW);
      }
    }
#if DEBUG_UART
    Serial.print(F(" - "));
    Serial.println(actualPattern);
#endif
    actualPattern++;                       // switch to next pattern for the next iteration
    if (actualPattern >= totalNoPattern) 
      actualPattern = 0;                   // wrap around
    previousMillis = millis();
  }
}

/* ***************************
    Setup
 * ************************* */

void setup() {
#if DEBUG_UART
  Serial.begin(115200);
  Serial.println(F("\nLarge LED pattern - with data in PROGMEM"));
#endif

  for (size_t i = 0; i < totalNoPins; i++) {
    pinMode(ledPin[i], OUTPUT);
  }
}

/* ***************************
    Main - Loop
 * ************************* */

void loop() {
  runSequence();
  // do what ever you want to do unblocked here
  
}
